@@ -1,6 +1,6 @@
 /*
  * ===========================================================================
- * (c) Copyright IBM Corp. 2018, 2025 All Rights Reserved
+ * (c) Copyright IBM Corp. 2018, 2024 All Rights Reserved
  * ===========================================================================
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,7 +32,6 @@ import jdk.internal.misc.Unsafe;
 import jdk.internal.ref.CleanerFactory;
 import jdk.internal.reflect.Reflection;
 import jdk.internal.reflect.CallerSensitive;
-import jdk.internal.util.StaticProperty;
 
 import sun.security.action.GetPropertyAction;
 
@@ -43,14 +42,12 @@ import openj9.internal.criu.InternalCRIUSupport;
 public class NativeCrypto {
 
     /* Define constants for the native digest algorithm indices. */
-    public static final int MD5 = 0;
-    public static final int SHA1_160 = 1;
-    public static final int SHA2_224 = 2;
-    public static final int SHA2_256 = 3;
-    public static final int SHA5_384 = 4;
-    public static final int SHA5_512 = 5;
-    public static final int SHA5_512_224 = 6;
-    public static final int SHA5_512_256 = 7;
+    public static final int SHA1_160 = 0;
+    public static final int SHA2_224 = 1;
+    public static final int SHA2_256 = 2;
+    public static final int SHA5_384 = 3;
+    public static final int SHA5_512 = 4;
+    public static final int MD5 = 5;
 
     /* Define constants for the EC field types. */
     public static final int ECField_Fp = 0;
@@ -84,41 +81,16 @@ public class NativeCrypto {
 
     private final boolean isOpenSSLFIPS;
 
-    @SuppressWarnings("restricted")
     private static long loadCryptoLibraries() {
         long osslVersion;
 
         try {
-            // Load jncrypto JNI library.
+            // load jncrypto JNI library
             System.loadLibrary("jncrypto");
-
-            // Get user-specified option to skip bundled OpenSSL library.
-            boolean skipBundled = Boolean.parseBoolean(
-                    GetPropertyAction.privilegedGetProperty("jdk.native.openssl.skipBundled"));
-
-            // Get user-specified OpenSSL library to use, if available.
-            String nativeLibName =
-                    GetPropertyAction.privilegedGetProperty("jdk.native.openssl.lib");
-
-            // Check that these mutually exclusive flags are not used at the same time.
-            if (skipBundled && (nativeLibName != null)) {
-                throw new RuntimeException("Conflicting properties " +
-                        "jdk.native.openssl.skipBundled and jdk.native.openssl.lib");
-            }
-
-            // Get the JDK location.
-            String javaHome = StaticProperty.javaHome();
-
-            // Load OpenSSL crypto library dynamically.
-            osslVersion = loadCrypto(traceEnabled, skipBundled, nativeLibName, javaHome);
-            if (osslVersion != -1) {
-                if (traceEnabled) {
-                    System.err.println("Native crypto library load succeeded - using native crypto library.");
-                }
-            } else {
-                if ((nativeLibName != null) && !nativeLibName.isEmpty()) {
-                    throw new RuntimeException(nativeLibName + " is not available, crypto libraries are not loaded");
-                }
+            // load OpenSSL crypto library dynamically
+            osslVersion = loadCrypto(traceEnabled);
+            if (traceEnabled && (osslVersion != -1)) {
+                System.err.println("Native crypto library load succeeded - using native crypto library.");
             }
         } catch (UnsatisfiedLinkError usle) {
             if (traceEnabled) {
@@ -126,7 +98,7 @@ public class NativeCrypto {
                 System.err.println("Warning: Native crypto library load failed." +
                         " Using Java crypto implementation.");
             }
-            // Signal load failure.
+            // signal load failure
             osslVersion = -1;
         }
         return osslVersion;
@@ -281,19 +253,14 @@ public class NativeCrypto {
         });
     }
 
-    /* OpenSSL utility interfaces */
+    /* Native digest interfaces */
 
-    private static final native long loadCrypto(boolean trace,
-                                                boolean skipBundled,
-                                                String libName,
-                                                String javaHome);
+    private static final native long loadCrypto(boolean trace);
 
     public static final native boolean isMD5Available();
 
     private static final native boolean isOpenSSLFIPS();
 
-    /* Native digest interfaces */
-
     public final native long DigestCreateContext(long nativeBuffer,
                                                  int algoIndex);
 
@@ -533,11 +500,4 @@ public class NativeCrypto {
                                               byte[] computedSecret,
                                               int computedSecretLength,
                                               int curveType);
-
-    /* Password based key derivation functions (PBKDF). */
-    public final native byte[] PBKDF2Derive(byte[] password,
-                                            byte[] salt,
-                                            int iterations,
-                                            int keyLength,
-                                            int hashAlgorithm);
 }
