@@ -1,6 +1,6 @@
 /*
  * ===========================================================================
- * (c) Copyright IBM Corp. 2022, 2025 All Rights Reserved
+ * (c) Copyright IBM Corp. 2022, 2024 All Rights Reserved
  * ===========================================================================
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -66,7 +66,7 @@ public final class RestrictedSecurity {
     private static final boolean allowSetProperties;
 
     private static final boolean isNSSSupported;
-    private static final boolean isOpenJCEPlusCertifiedPlatform;
+    private static final boolean isOpenJCEPlusSupported;
 
     private static final boolean userSetProfile;
     private static final boolean shouldEnableSecurity;
@@ -79,13 +79,20 @@ public final class RestrictedSecurity {
 
     private static ProfileParser profileParser;
 
-    private static boolean enableCheckHashes;
-
     private static RestrictedSecurityProperties restricts;
 
     private static final Set<String> unmodifiableProperties = new HashSet<>();
 
+    private static final Map<String, List<String>> supportedPlatformsNSS = new HashMap<>();
+    private static final Map<String, List<String>> supportedPlatformsOpenJCEPlus = new HashMap<>();
+
     static {
+        supportedPlatformsNSS.put("Arch", List.of("amd64", "ppc64le", "s390x"));
+        supportedPlatformsNSS.put("OS", List.of("Linux"));
+
+        supportedPlatformsOpenJCEPlus.put("Arch", List.of("amd64", "ppc64", "s390x"));
+        supportedPlatformsOpenJCEPlus.put("OS", List.of("Linux", "AIX", "Windows"));
+
         @SuppressWarnings("removal")
         String[] props = AccessController.doPrivileged(
                 new PrivilegedAction<>() {
@@ -98,19 +105,37 @@ public final class RestrictedSecurity {
                                 System.getProperty("semeru.fips.allowsetproperties") };
                     }
                 });
-        final List<String> nssSupportedArch = List.of("amd64", "ppc64le", "s390x");
-        final List<String> openjceplusCertifiedArch = List.of("amd64", "ppc64", "s390x");
-        final List<String> openjceplusCertifiedOS = List.of("AIX", "Linux", "Windows");
-        String osName = props[2];
-        String osArch = props[3];
 
+        boolean isOsSupported, isArchSupported;
         // Check whether the NSS FIPS solution is supported.
-        isNSSSupported = osName.contains("Linux")
-                && containsAny(osArch, nssSupportedArch);
+        isOsSupported = false;
+        for (String os: supportedPlatformsNSS.get("OS")) {
+            if (props[2].contains(os)) {
+                isOsSupported = true;
+            }
+        }
+        isArchSupported = false;
+        for (String arch: supportedPlatformsNSS.get("Arch")) {
+            if (props[3].contains(arch)) {
+                isArchSupported = true;
+            }
+        }
+        isNSSSupported = isOsSupported && isArchSupported;
 
-        // Check whether the OpenJCEPlus FIPS solution is certified.
-        isOpenJCEPlusCertifiedPlatform = containsAny(osName, openjceplusCertifiedOS)
-                && containsAny(osArch, openjceplusCertifiedArch);
+        // Check whether the OpenJCEPlus FIPS solution is supported.
+        isOsSupported = false;
+        for (String os: supportedPlatformsOpenJCEPlus.get("OS")) {
+            if (props[2].contains(os)) {
+                isOsSupported = true;
+            }
+        }
+        isArchSupported = false;
+        for (String arch: supportedPlatformsOpenJCEPlus.get("Arch")) {
+            if (props[3].contains(arch)) {
+                isArchSupported = true;
+            }
+        }
+        isOpenJCEPlusSupported = isOsSupported && isArchSupported;
 
         // Check the default solution to see if FIPS is supported.
         isFIPSSupported = isNSSSupported;
@@ -147,18 +172,6 @@ public final class RestrictedSecurity {
         super();
     }
 
-    private static boolean isJarVerifierInStackTrace() {
-        java.util.function.Predicate<Class<?>> isJarVerifier =
-                clazz -> "java.util.jar.JarVerifier".equals(clazz.getName())
-                      && "java.base".equals(clazz.getModule().getName());
-
-        java.util.function.Function<Stream<StackWalker.StackFrame>, Boolean> matcher =
-                stream -> stream.map(StackWalker.StackFrame::getDeclaringClass)
-                                .anyMatch(isJarVerifier);
-
-        return StackWalker.getInstance(StackWalker.Option.RETAIN_CLASS_REFERENCE).walk(matcher);
-    }
-
     /**
      * Check loaded profiles' hash values.
      *
@@ -167,28 +180,9 @@ public final class RestrictedSecurity {
      * is calculated and compared to the expected value.
      */
     public static void checkHashValues() {
-        checkHashValues(true);
-    }
-
-    @SuppressWarnings("removal")
-    private static void checkHashValues(boolean fromProviders) {
-        ProfileParser parser = profileParser;
-        if (parser != null) {
-            if (fromProviders) {
-                enableCheckHashes = true;
-            }
-            if (enableCheckHashes) {
-                boolean isVerifying;
-                if (System.getSecurityManager() == null) {
-                    isVerifying = isJarVerifierInStackTrace();
-                } else {
-                    isVerifying = AccessController.doPrivileged((PrivilegedAction<Boolean>)(() -> isJarVerifierInStackTrace()));
-                }
-                if (!isVerifying) {
-                    profileParser = null;
-                    parser.checkHashValues();
-                }
-            }
+        if (profileParser != null) {
+            profileParser.checkHashValues();
+            profileParser = null;
         }
     }
 
@@ -254,29 +248,14 @@ public final class RestrictedSecurity {
     }
 
     /**
-     * Check if the service is allowed to be used in restricted security mode.
+     * Check if the service is allowed in restricted security mode.
      *
      * @param service the service to check
-     * @return true if the service is allowed to be used
+     * @return true if the service is allowed
      */
     public static boolean isServiceAllowed(Service service) {
         if (securityEnabled) {
-            checkHashValues(false);
-            return restricts.isRestrictedServiceAllowed(service, true);
-        }
-        return true;
-    }
-
-    /**
-     * Check if the service is allowed to be registered in restricted security mode.
-     *
-     * @param service the service to check
-     * @return true if the service is allowed to be registered
-     */
-    public static boolean canServiceBeRegistered(Service service) {
-        if (securityEnabled) {
-            checkHashValues(false);
-            return restricts.isRestrictedServiceAllowed(service, false);
+            return restricts.isRestrictedServiceAllowed(service);
         }
         return true;
     }
@@ -289,7 +268,6 @@ public final class RestrictedSecurity {
      */
     public static boolean isProviderAllowed(String providerName) {
         if (securityEnabled) {
-            checkHashValues(false);
             // Remove argument, e.g. -NSS-FIPS, if present.
             int pos = providerName.indexOf('-');
             if (pos >= 0) {
@@ -309,7 +287,6 @@ public final class RestrictedSecurity {
      */
     public static boolean isProviderAllowed(Class<?> providerClazz) {
         if (securityEnabled) {
-            checkHashValues(false);
             String providerClassName = providerClazz.getName();
 
             // Check if the specified class extends java.security.Provider.
@@ -405,14 +382,12 @@ public final class RestrictedSecurity {
                     + " on this platform.");
         }
 
+        if (profileID.contains("OpenJCEPlus") && !isOpenJCEPlusSupported) {
+            printStackTraceAndExit("OpenJCEPlus RestrictedSecurity profiles are not supported"
+                    + " on this platform.");
+        }
+
         if (debug != null) {
-            if (profileID.contains("OpenJCEPlus")) {
-                if (isOpenJCEPlusCertifiedPlatform) {
-                    debug.println("OpenJCEPlus RestrictedSecurity profile running on a certified platform.");
-                } else {
-                    debug.println("OpenJCEPlus RestrictedSecurity profile running in developer mode.");
-                }
-            }
             debug.println("RestrictedSecurity profile " + profileID
                     + " is supported on this platform.");
         }
@@ -581,17 +556,11 @@ public final class RestrictedSecurity {
         propsMapping.put("jdk.tls.legacyAlgorithms", restricts.jdkTlsLegacyAlgorithms);
         propsMapping.put("jdk.certpath.disabledAlgorithms", restricts.jdkCertpathDisabledAlgorithms);
         propsMapping.put("jdk.security.legacyAlgorithms", restricts.jdkSecurityLegacyAlgorithms);
-
-        if (restricts.descIsFIPS) {
-            if (restricts.jdkFipsMode == null) {
-                printStackTraceAndExit(profileID + ".fips.mode property is not set in FIPS profile");
-            }
-            String fipsMode = System.getProperty("com.ibm.fips.mode");
-            if (fipsMode == null) {
-                System.setProperty("com.ibm.fips.mode", restricts.jdkFipsMode);
-            } else if (!fipsMode.equals(restricts.jdkFipsMode)) {
-                printStackTraceAndExit("Property com.ibm.fips.mode is incompatible with semeru.customprofile and semeru.fips properties");
-            }
+        String fipsMode = System.getProperty("com.ibm.fips.mode");
+        if (fipsMode == null) {
+            System.setProperty("com.ibm.fips.mode", restricts.jdkFipsMode);
+        } else if (!fipsMode.equals(restricts.jdkFipsMode)) {
+            printStackTraceAndExit("Property com.ibm.fips.mode is incompatible with semeru.customprofile and semeru.fips properties");
         }
 
         if (userEnabledFIPS && !allowSetProperties) {
@@ -704,15 +673,6 @@ public final class RestrictedSecurity {
         return "*".equals(string);
     }
 
-    private static boolean containsAny(String string, List<String> substrings) {
-        for (String substring : substrings) {
-            if (string.contains(substring)) {
-                return true;
-            }
-        }
-        return false;
-    }
-
     /**
      * This class is used to save and operate on restricted security
      * properties which are loaded from the java.security file.
@@ -796,11 +756,10 @@ public final class RestrictedSecurity {
         /**
          * Check if the Service is allowed in restricted security mode.
          *
-         * @param service   the Service to check
-         * @param checkUse  should its attempted use be checked against the accepted
+         * @param service the Service to check
          * @return true if the Service is allowed
          */
-        boolean isRestrictedServiceAllowed(Service service, boolean checkUse) {
+        boolean isRestrictedServiceAllowed(Service service) {
             Provider provider = service.getProvider();
             String providerClassName = provider.getClass().getName();
 
@@ -832,18 +791,15 @@ public final class RestrictedSecurity {
             if (debug != null) {
                 debug.println("Security constraints check of provider.");
             }
-            constraints:
             for (Constraint constraint : constraints) {
                 String cType = constraint.type;
                 String cAlgorithm = constraint.algorithm;
                 String cAttribute = constraint.attributes;
-                String cAcceptedUses = constraint.acceptedUses;
                 if (debug != null) {
                     debug.println("Checking provider constraint:"
                                 + "\n\tService type: " + cType
                                 + "\n\tAlgorithm: " + cAlgorithm
-                                + "\n\tAttributes: " + cAttribute
-                                + "\n\tAccepted uses: " + cAcceptedUses);
+                                + "\n\tAttributes: " + cAttribute);
                 }
 
                 if (!isAsterisk(cType) && !type.equals(cType)) {
@@ -851,125 +807,66 @@ public final class RestrictedSecurity {
                     if (debug != null) {
                         debug.println("The constraint doesn't apply to the service type.");
                     }
-                    continue constraints;
+                    continue;
                 }
                 if (!isAsterisk(cAlgorithm) && !algorithm.equalsIgnoreCase(cAlgorithm)) {
                     // The constraint doesn't apply to the service algorithm.
                     if (debug != null) {
                         debug.println("The constraint doesn't apply to the service algorithm.");
                     }
-                    continue constraints;
+                    continue;
+                }
+
+                // For type and algorithm match, and attribute is *.
+                if (isAsterisk(cAttribute)) {
+                    if (debug != null) {
+                        debug.println("The following service:"
+                                + "\n\tService type: " + type
+                                + "\n\tAlgorithm: " + algorithm
+                                + "\nis allowed in provider: " + providerClassName);
+                    }
+                    return true;
                 }
 
                 // For type and algorithm match, and attribute is not *.
                 // Then continue checking attributes.
-                if (!isAsterisk(cAttribute)) {
-                    String[] cAttributeArray = cAttribute.split(":");
+                String[] cAttributeArray = cAttribute.split(":");
 
-                    // For each attribute, must be all matched for return allowed.
-                    for (String attribute : cAttributeArray) {
-                        String[] input = attribute.split("=", 2);
+                // For each attribute, must be all matched for return allowed.
+                for (String attribute : cAttributeArray) {
+                    String[] input = attribute.split("=", 2);
 
-                        String cName = input[0].trim();
-                        String cValue = input[1].trim();
-                        String sValue = service.getAttribute(cName);
-                        if (debug != null) {
-                            debug.println("Checking specific attribute with:"
-                                    + "\n\tName: " + cName
-                                    + "\n\tValue: " + cValue
-                                    + "\nagainst the service attribute value: " + sValue);
-                        }
-                        if ((sValue == null) || !cValue.equalsIgnoreCase(sValue)) {
-                            // If any of the attributes don't match,
-                            // then this constraint doesn't match so move on.
-                            if (debug != null) {
-                                debug.println("Attributes don't match!");
-                                debug.println("The following service:"
-                                            + "\n\tService type: " + type
-                                            + "\n\tAlgorithm: " + algorithm
-                                            + "\n\tAttribute: " + cAttribute
-                                            + "\nis NOT allowed in provider: " + providerClassName);
-                            }
-                            continue constraints;
-                        }
-                        if (debug != null) {
-                            debug.println("Attributes match!");
-                        }
+                    String cName = input[0].trim();
+                    String cValue = input[1].trim();
+                    String sValue = service.getAttribute(cName);
+                    if (debug != null) {
+                        debug.println("Checking specific attribute with:"
+                                + "\n\tName: " + cName
+                                + "\n\tValue: " + cValue
+                                + "\nagainst the service attribute value: " + sValue);
                     }
-                }
-
-                // See if accepted uses have been specified and apply
-                // them to the call stack.
-                if (checkUse && (cAcceptedUses != null)) {
-                    String[] optionAndValue = cAcceptedUses.split(":");
-                    String option = optionAndValue[0];
-                    String value = optionAndValue[1];
-                    StackTraceElement[] stackElements = Thread.currentThread().getStackTrace();
-                    boolean found = false;
-                    for (StackTraceElement stackElement : stackElements) {
+                    if ((sValue == null) || !cValue.equalsIgnoreCase(sValue)) {
+                        // If any attribute doesn't match, return service is not allowed.
                         if (debug != null) {
-                            debug.println("Attempting to match " + stackElement + " with: " + option + " : " + value);
-                        }
-                        String stackElemModule = stackElement.getModuleName();
-                        String stackElemFullClassName = stackElement.getClassName();
-                        int stackElemEnd = stackElemFullClassName.lastIndexOf('.');
-                        String stackElemPackage = null;
-                        if (stackElemEnd != -1) {
-                            stackElemPackage = stackElemFullClassName.substring(0, stackElemEnd);
-                        }
-                        String module;
-                        switch (option) {
-                        case "ModuleAndFullClassName":
-                            String[] moduleAndFullClassName = value.split("/");
-                            module = moduleAndFullClassName[0];
-                            String fullClassName = moduleAndFullClassName[1];
-                            found = module.equals(stackElemModule) && stackElemFullClassName.equals(fullClassName);
-                            break;
-                        case "ModuleAndPackage":
-                            String[] moduleAndPackage = value.split("/");
-                            module = moduleAndPackage[0];
-                            String packageValue = moduleAndPackage[1];
-                            found = module.equals(stackElemModule) && packageValue.equals(stackElemPackage);
-                            break;
-                        case "FullClassName":
-                            found = stackElemFullClassName.equals(value);
-                            break;
-                        case "Package":
-                            found = value.equals(stackElemPackage);
-                            break;
-                        default:
-                            printStackTraceAndExit("Incorrect option to match in constraint: " + constraint);
-                            break;
-                        }
-
-                        if (found) {
-                            break;
-                        }
-                    }
-
-                    // If nothing matching the accepted uses is found in the call stack,
-                    // then this constraint doesn't match so move on.
-                    if (!found) {
-                        if (debug != null) {
-                            debug.println("Classes in call stack are not part of accepted uses!");
+                            debug.println("Attributes don't match!");
                             debug.println("The following service:"
                                         + "\n\tService type: " + type
                                         + "\n\tAlgorithm: " + algorithm
                                         + "\n\tAttribute: " + cAttribute
-                                        + "\n\tAccepted uses: " + cAcceptedUses
                                         + "\nis NOT allowed in provider: " + providerClassName);
                         }
-                        continue constraints;
+                        return false;
+                    }
+                    if (debug != null) {
+                        debug.println("Attributes match!");
                     }
                 }
-
                 if (debug != null) {
                     debug.println("All attributes matched!");
                     debug.println("The following service:"
                                 + "\n\tService type: " + type
                                 + "\n\tAlgorithm: " + algorithm
                                 + "\n\tAttribute: " + cAttribute
-                                + "\n\tAccepted uses: " + cAcceptedUses
                                 + "\nis allowed in provider: " + providerClassName);
                 }
                 return true;
@@ -1554,10 +1451,9 @@ public final class RestrictedSecurity {
 
             // Check whether constraints are properly specified.
             final String typeRE = "\\w+";
-            final String algoRE = "[A-Za-z0-9./_:#-]+";
+            final String algoRE = "[A-Za-z0-9./_-]+";
             final String attrRE = "[A-Za-z0-9=*|.:]+";
-            final String usesRE = "[A-Za-z0-9._:/$]+";
-            final String consRE = "\\{(" + typeRE + "),(" + algoRE + "),(" + attrRE + ")(," + usesRE + ")?\\}";
+            final String consRE = "\\{(" + typeRE + "),(" + algoRE + "),(" + attrRE + ")\\}";
             p = Pattern.compile(
                 "\\["
                 + "([+-]?)"             // option to append or remove
@@ -1580,42 +1476,6 @@ public final class RestrictedSecurity {
                 String inType = m.group(1);
                 String inAlgorithm = m.group(2);
                 String inAttributes = m.group(3);
-                String inAcceptedUses = m.group(4);
-
-                if (inAcceptedUses != null) {
-                    inAcceptedUses = inAcceptedUses.substring(1);
-                    boolean isSpecIncorrect = false;
-                    String[] optionAndValue = inAcceptedUses.split(":");
-                    if (optionAndValue.length != 2) {
-                        isSpecIncorrect = true;
-                    } else {
-                        String option = optionAndValue[0];
-                        String value = optionAndValue[1];
-                        switch (option) {
-                        case "ModuleAndFullClassName":
-                            if (value.split("/").length != 2) {
-                                isSpecIncorrect = true;
-                            }
-                            break;
-                        case "ModuleAndPackage":
-                            if (value.split("/").length != 2) {
-                                isSpecIncorrect = true;
-                            }
-                            break;
-                        case "FullClassName":
-                        case "Package":
-                            // Nothing further to check in those options.
-                            break;
-                        default:
-                            isSpecIncorrect = true;
-                            break;
-                        }
-                    }
-                    if (isSpecIncorrect) {
-                        printStackTraceAndExit("Incorrect specification of accepted uses in constraint for "
-                                + inType + ", " + inAlgorithm + ": " + inAcceptedUses);
-                    }
-                }
 
                 // Each attribute must includes 2 fields (key and value) or *.
                 if (!isAsterisk(inAttributes)) {
@@ -1628,8 +1488,7 @@ public final class RestrictedSecurity {
                         }
                     }
                 }
-
-                Constraint constraint = new Constraint(inType, inAlgorithm, inAttributes, inAcceptedUses);
+                Constraint constraint = new Constraint(inType, inAlgorithm, inAttributes);
                 constraints.add(constraint);
             }
 
@@ -1900,7 +1759,7 @@ public final class RestrictedSecurity {
                 + "(\\["                                // constraints [optional]
                     + "\\s*"
                     + "([+-])?"                         // action [optional]
-                    + "[A-Za-z0-9{}.=*|:$#,/_\\s-]+"    // constraint definition
+                    + "[A-Za-z0-9{}.=*|:,/_\\s-]+"      // constraint definition
                 + "\\])?"
                 + "\\s*"
                 + "$");
@@ -1914,8 +1773,8 @@ public final class RestrictedSecurity {
 
                 String action = m.group(4);
                 if (!update && !isNullOrBlank(action)) {
-                    printStackTraceAndExit("Constraints of provider not previously specified"
-                            + " cannot be modified: " + providerName);
+                    printStackTraceAndExit("You cannot add or remove to provider "
+                            + m.group(1) + ". This is the base profile.");
                 }
             } else {
                 printStackTraceAndExit("Provider format is incorrect: " + providerInfo);
@@ -1963,27 +1822,17 @@ public final class RestrictedSecurity {
         final String type;
         final String algorithm;
         final String attributes;
-        final String acceptedUses;
 
-        Constraint(String type, String algorithm, String attributes, String acceptedUses) {
+        Constraint(String type, String algorithm, String attributes) {
             super();
             this.type = type;
             this.algorithm = algorithm;
             this.attributes = attributes;
-            this.acceptedUses = acceptedUses;
         }
 
         @Override
         public String toString() {
-            StringBuilder buffer = new StringBuilder();
-            buffer.append("{").append(type);
-            buffer.append(", ").append(algorithm);
-            buffer.append(", ").append(attributes);
-            if (acceptedUses != null) {
-                buffer.append(", ").append(acceptedUses);
-            }
-            buffer.append("}");
-            return buffer.toString();
+            return "{" + type + ", " + algorithm + ", " + attributes + "}";
         }
 
         @Override
@@ -1994,15 +1843,14 @@ public final class RestrictedSecurity {
             if (obj instanceof Constraint other) {
                 return Objects.equals(type, other.type)
                     && Objects.equals(algorithm, other.algorithm)
-                    && Objects.equals(attributes, other.attributes)
-                    && Objects.equals(acceptedUses, other.acceptedUses);
+                    && Objects.equals(attributes, other.attributes);
             }
             return false;
         }
 
         @Override
         public int hashCode() {
-            return Objects.hash(type, algorithm, attributes, acceptedUses);
+            return Objects.hash(type, algorithm, attributes);
         }
     }
 }
