@@ -278,11 +278,10 @@ abstract class ClassSpecializer<T,K,S extends ClassSpecializer<T,K,S>.SpeciesDat
 
         @Override
         public boolean equals(Object obj) {
-            if (!(obj instanceof ClassSpecializer.SpeciesData)) {
+            if (!(obj instanceof ClassSpecializer<?, ?, ?>.SpeciesData that)) {
                 return false;
             }
-            @SuppressWarnings("rawtypes")
-            ClassSpecializer.SpeciesData that = (ClassSpecializer.SpeciesData) obj;
+
             return this.outer() == that.outer() && this.key.equals(that.key);
         }
 
@@ -669,8 +668,8 @@ abstract class ClassSpecializer<T,K,S extends ClassSpecializer<T,K,S>.SpeciesDat
                     for (X x : types) {
                         String vn = name;
                         Class<?> vt;
-                        if (x instanceof Class) {
-                            vt = (Class<?>) x;
+                        if (x instanceof Class<?> cl) {
+                            vt = cl;
                             // make the names friendlier if debugging
                             assert((vn = vn + "_" + (i++)) != null);
                         } else {
