@@ -25,7 +25,7 @@
 
 /*
  * ===========================================================================
- * (c) Copyright IBM Corp. 2022, 2025 All Rights Reserved
+ * (c) Copyright IBM Corp. 2022, 2024 All Rights Reserved
  * ===========================================================================
  */
 
@@ -124,19 +124,13 @@ public final class Security {
         if ("true".equalsIgnoreCase(props.getProperty
                 ("security.overridePropertiesFile"))) {
 
-            String extraPropList = System.getProperty
-                    ("java.security.propertiesList");
-            if ((extraPropList != null) && !extraPropList.isBlank()) {
-                loadPropsList(extraPropList);
-            } else {
-                String extraPropFile = System.getProperty
-                        ("java.security.properties");
-                if (extraPropFile != null && extraPropFile.startsWith("=")) {
-                    overrideAll = true;
-                    extraPropFile = extraPropFile.substring(1);
-                }
-                loadProps(null, extraPropFile, overrideAll);
+            String extraPropFile = System.getProperty
+                    ("java.security.properties");
+            if (extraPropFile != null && extraPropFile.startsWith("=")) {
+                overrideAll = true;
+                extraPropFile = extraPropFile.substring(1);
             }
+            loadProps(null, extraPropFile, overrideAll);
         }
         initialSecurityProperties = (Properties) props.clone();
         if (sdebug != null) {
@@ -162,47 +156,6 @@ public final class Security {
         }
     }
 
-    private static void loadPropsList(String extraPropList) {
-        for (String file : extraPropList.split(File.pathSeparator)) {
-            InputStream is = null;
-            try {
-                if (file.startsWith("=")) {
-                    throw new IllegalArgumentException(
-                            "java.security.propertiesList does not support '=' prefix: " + file);
-                }
-                file = PropertyExpander.expand(file);
-                File propFile = new File(file);
-                URL propURL;
-                if (propFile.exists()) {
-                    propURL = newURL("file:" + propFile.getCanonicalPath());
-                } else {
-                    propURL = newURL(file);
-                }
-                is = propURL.openStream();
-                props.load(is);
-                if (sdebug != null) {
-                    sdebug.println("reading security properties file: " + file);
-                }
-            } catch (IOException | PropertyExpander.ExpandException e) {
-                if (sdebug != null) {
-                    sdebug.println("unable to load security properties from " + file);
-                    e.printStackTrace();
-                }
-                break;
-            } finally {
-                if (is != null) {
-                    try {
-                        is.close();
-                    } catch (IOException ioe) {
-                        if (sdebug != null) {
-                            sdebug.println("unable to close input stream");
-                        }
-                    }
-                }
-            }
-        }
-    }
-
     private static boolean loadProps(File masterFile, String extraPropFile, boolean overrideAll) {
         InputStream is = null;
         try {
@@ -373,7 +326,10 @@ public final class Security {
      * Adds a new provider, at a specified position. The position is
      * the preference order in which providers are searched for
      * requested algorithms.  The position is 1-based, that is,
-     * 1 is most preferred, followed by 2, and so on.
+     * 1 is most preferred, followed by 2, and so on.  If the position
+     * is less than 1 or greater than n, where n is the number of installed
+     * providers, the provider (if not already installed) is inserted at
+     * the end of the list, or at the n + 1 position.
      *
      * <p>If the given provider is installed at the requested position,
      * the provider that used to be at that position, and all providers
@@ -411,26 +367,14 @@ public final class Security {
      */
     public static synchronized int insertProviderAt(Provider provider,
             int position) {
-        String providerName = provider.getName();
-        checkInsertProvider(providerName);
-        ProviderList list = Providers.getFullProviderList();
 
         /*[IF CRIU_SUPPORT]*/
-        if (InternalCRIUSupport.enableCRIUSecProvider()) {
-            for (Provider existingProvider : list.providers()) {
-                if ("CRIUSEC".equals(existingProvider.getName())) {
-                    if (criuDebug) {
-                        System.out.println("Trying to insert + " + providerName
-                                + " during the pre-checkpoint which is not allowed.");
-                    }
-                    throw new RuntimeException("Inserting " + providerName
-                            + " during the pre-checkpoint is not allowed");
-                }
-            }
-        }
         CRIUConfigurator.invalidateAlgorithmCache();
         /*[ENDIF] CRIU_SUPPORT */
 
+        String providerName = provider.getName();
+        checkInsertProvider(providerName);
+        ProviderList list = Providers.getFullProviderList();
         ProviderList newList = ProviderList.insertAt(list, provider, position - 1);
         if (list == newList) {
             return -1;
