@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1995, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -86,7 +86,7 @@ import openj9.internal.criu.NotCheckpointSafe;
 /*[ENDIF] CRIU_SUPPORT */
 
 /**
- * This class is used to read entries from a zip file.
+ * This class is used to read entries from a ZIP file.
  *
  * <p> Unless otherwise noted, passing a {@code null} argument to a constructor
  * or method in this class will cause a {@link NullPointerException} to be
@@ -105,26 +105,27 @@ import openj9.internal.criu.NotCheckpointSafe;
  */
 public class ZipFile implements ZipConstants, Closeable {
 
-    private final String name;     // zip file name
+    private final String filePath;     // ZIP file path
+    private final String fileName;     // name of the file
     private volatile boolean closeRequested;
 
-    // The "resource" used by this zip file that needs to be
+    // The "resource" used by this ZIP file that needs to be
     // cleaned after use.
     // a) the input streams that need to be closed
     // b) the list of cached Inflater objects
-    // c) the "native" source of this zip file.
+    // c) the "native" source of this ZIP file.
     private final @Stable CleanableResource res;
 
     private static final int STORED = ZipEntry.STORED;
     private static final int DEFLATED = ZipEntry.DEFLATED;
 
     /**
-     * Mode flag to open a zip file for reading.
+     * Mode flag to open a ZIP file for reading.
      */
     public static final int OPEN_READ = 0x1;
 
     /**
-     * Mode flag to open a zip file and mark it for deletion.  The file will be
+     * Mode flag to open a ZIP file and mark it for deletion.  The file will be
      * deleted some time between the moment that it is opened and the moment
      * that it is closed, but its contents will remain accessible via the
      * {@code ZipFile} object until either the close method is invoked or the
@@ -140,7 +141,7 @@ public class ZipFile implements ZipConstants, Closeable {
             getDisableZip64ExtraFieldValidation();
 
     /**
-     * Opens a zip file for reading.
+     * Opens a ZIP file for reading.
      *
      * <p>First, if there is a security manager, its {@code checkRead}
      * method is called with the {@code name} argument as its argument
@@ -149,7 +150,7 @@ public class ZipFile implements ZipConstants, Closeable {
      * <p>The UTF-8 {@link java.nio.charset.Charset charset} is used to
      * decode the entry names and comments.
      *
-     * @param name the name of the zip file
+     * @param name the name of the ZIP file
      * @throws ZipException if a ZIP format error has occurred
      * @throws IOException if an I/O error has occurred
      * @throws SecurityException if a security manager exists and its
@@ -236,6 +237,7 @@ public class ZipFile implements ZipConstants, Closeable {
      *
      * @since 1.7
      */
+    @SuppressWarnings("this-escape")
     public ZipFile(File file, int mode, Charset charset) throws IOException
     {
         if (((mode & OPEN_READ) == 0) ||
@@ -255,7 +257,8 @@ public class ZipFile implements ZipConstants, Closeable {
         }
         Objects.requireNonNull(charset, "charset");
 
-        this.name = name;
+        this.filePath = name;
+        this.fileName = file.getName();
         long t0 = System.nanoTime();
 
         this.res = new CleanableResource(this, ZipCoder.get(charset), file, mode);
@@ -265,13 +268,13 @@ public class ZipFile implements ZipConstants, Closeable {
     }
 
     /**
-     * Opens a zip file for reading.
+     * Opens a ZIP file for reading.
      *
      * <p>First, if there is a security manager, its {@code checkRead}
      * method is called with the {@code name} argument as its argument
      * to ensure the read is allowed.
      *
-     * @param name the name of the zip file
+     * @param name the name of the ZIP file
      * @param charset
      *        the {@linkplain java.nio.charset.Charset charset} to
      *        be used to decode the ZIP entry name and comment that are not
@@ -315,11 +318,13 @@ public class ZipFile implements ZipConstants, Closeable {
     }
 
     /**
-     * Returns the zip file comment, or null if none.
+     * Returns the ZIP file comment. If a comment does not exist or an error is
+     * encountered decoding the comment using the charset specified
+     * when opening the ZIP file, then {@code null} is returned.
      *
-     * @return the comment string for the zip file, or null if none
+     * @return the comment string for the ZIP file, or null if none
      *
-     * @throws IllegalStateException if the zip file has been closed
+     * @throws IllegalStateException if the ZIP file has been closed
      *
      * @since 1.7
      */
@@ -329,17 +334,23 @@ public class ZipFile implements ZipConstants, Closeable {
             if (res.zsrc.comment == null) {
                 return null;
             }
-            return res.zsrc.zc.toString(res.zsrc.comment);
+            // If there is a problem decoding the byte array which represents
+            // the ZIP file comment, return null;
+            try {
+                return res.zsrc.zc.toString(res.zsrc.comment);
+            } catch (IllegalArgumentException iae) {
+                return null;
+            }
         }
     }
 
     /**
-     * Returns the zip file entry for the specified name, or null
+     * Returns the ZIP file entry for the specified name, or null
      * if not found.
      *
      * @param name the name of the entry
-     * @return the zip file entry, or null if not found
-     * @throws IllegalStateException if the zip file has been closed
+     * @return the ZIP file entry, or null if not found
+     * @throws IllegalStateException if the ZIP file has been closed
      */
     /*[IF CRIU_SUPPORT]*/
     @NotCheckpointSafe
@@ -359,7 +370,7 @@ public class ZipFile implements ZipConstants, Closeable {
 
     /**
      * Returns an input stream for reading the contents of the specified
-     * zip file entry.
+     * ZIP file entry.
      * <p>
      * Closing this ZIP file will, in turn, close all input streams that
      * have been returned by invocations of this method.
@@ -370,13 +381,13 @@ public class ZipFile implements ZipConstants, Closeable {
      * read(byte[], int, int)} method can modify any element of the output
      * buffer.
      *
-     * @param entry the zip file entry
+     * @param entry the ZIP file entry
      * @return the input stream for reading the contents of the specified
-     * zip file entry or null if the zip file entry does not exist
-     * within the zip file.
+     * ZIP file entry or null if the ZIP file entry does not exist
+     * within the ZIP file.
      * @throws ZipException if a ZIP format error has occurred
      * @throws IOException if an I/O error has occurred
-     * @throws IllegalStateException if the zip file has been closed
+     * @throws IllegalStateException if the ZIP file has been closed
      */
     public InputStream getInputStream(ZipEntry entry) throws IOException {
         Objects.requireNonNull(entry, "entry");
@@ -496,7 +507,16 @@ public class ZipFile implements ZipConstants, Closeable {
      * @return the path name of the ZIP file
      */
     public String getName() {
-        return name;
+        return filePath;
+    }
+
+    /**
+     * {@return a string identifying this {@code ZipFile}, for debugging}
+     */
+    @Override
+    public String toString() {
+        return this.fileName
+                + "@" + Integer.toHexString(System.identityHashCode(this));
     }
 
     private class ZipEntryIterator<T extends ZipEntry>
@@ -546,7 +566,7 @@ public class ZipFile implements ZipConstants, Closeable {
     /**
      * Returns an enumeration of the ZIP file entries.
      * @return an enumeration of the ZIP file entries
-     * @throws IllegalStateException if the zip file has been closed
+     * @throws IllegalStateException if the ZIP file has been closed
      */
     public Enumeration<? extends ZipEntry> entries() {
         synchronized (this) {
@@ -598,7 +618,7 @@ public class ZipFile implements ZipConstants, Closeable {
      * the central directory of the ZIP file.
      *
      * @return an ordered {@code Stream} of entries in this ZIP file
-     * @throws IllegalStateException if the zip file has been closed
+     * @throws IllegalStateException if the ZIP file has been closed
      * @since 1.8
      */
     public Stream<? extends ZipEntry> stream() {
@@ -617,13 +637,13 @@ public class ZipFile implements ZipConstants, Closeable {
     }
 
     /*
-     * Returns an ordered {@code Stream} over the zip file entry names.
+     * Returns an ordered {@code Stream} over the ZIP file entry names.
      *
      * Entry names appear in the {@code Stream} in the order they appear in
      * the central directory of the ZIP file.
      *
-     * @return an ordered {@code Stream} of entry names in this zip file
-     * @throws IllegalStateException if the zip file has been closed
+     * @return an ordered {@code Stream} of entry names in this ZIP file
+     * @throws IllegalStateException if the ZIP file has been closed
      * @since 10
      */
     private Stream<String> entryNameStream() {
@@ -635,13 +655,13 @@ public class ZipFile implements ZipConstants, Closeable {
     }
 
     /*
-     * Returns an ordered {@code Stream} over the zip file entries.
+     * Returns an ordered {@code Stream} over the ZIP file entries.
      *
      * Entries appear in the {@code Stream} in the order they appear in
      * the central directory of the jar file.
      *
-     * @return an ordered {@code Stream} of entries in this zip file
-     * @throws IllegalStateException if the zip file has been closed
+     * @return an ordered {@code Stream} of entries in this ZIP file
+     * @throws IllegalStateException if the ZIP file has been closed
      * @since 10
      */
     private Stream<JarEntry> jarStream() {
@@ -710,7 +730,7 @@ public class ZipFile implements ZipConstants, Closeable {
      * Returns the number of entries in the ZIP file.
      *
      * @return the number of entries in the ZIP file
-     * @throws IllegalStateException if the zip file has been closed
+     * @throws IllegalStateException if the ZIP file has been closed
      */
     public int size() {
         synchronized (this) {
@@ -810,7 +830,7 @@ public class ZipFile implements ZipConstants, Closeable {
                 }
             }
 
-            // Release zip src
+            // Release ZIP src
             if (zsrc != null) {
                 synchronized (zsrc) {
                     try {
@@ -845,7 +865,7 @@ public class ZipFile implements ZipConstants, Closeable {
 
         synchronized (this) {
             // Close streams, release their inflaters, release cached inflaters
-            // and release zip source
+            // and release ZIP source
             try {
                 res.clean();
             } catch (UncheckedIOException ioe) {
@@ -871,7 +891,7 @@ public class ZipFile implements ZipConstants, Closeable {
 
     /*
      * Inner class implementing the input stream used to read a
-     * (possibly compressed) zip file entry.
+     * (possibly compressed) ZIP file entry.
      */
     private class ZipFileInputStream extends InputStream {
         private volatile boolean closeRequested;
@@ -884,7 +904,7 @@ public class ZipFile implements ZipConstants, Closeable {
             rem = CENSIZ(cen, cenpos);
             size = CENLEN(cen, cenpos);
             pos = CENOFF(cen, cenpos);
-            // zip64
+            // ZIP64
             if (rem == ZIP64_MAGICVAL || size == ZIP64_MAGICVAL ||
                 pos == ZIP64_MAGICVAL) {
                 checkZIP64(cen, cenpos);
@@ -931,7 +951,7 @@ public class ZipFile implements ZipConstants, Closeable {
         }
 
         /*
-         * The Zip file spec explicitly allows the LOC extra data size to
+         * The ZIP file spec explicitly allows the LOC extra data size to
          * be different from the CEN extra data size. Since we cannot trust
          * the CEN extra data size, we need to read the LOC to determine
          * the entry data offset.
@@ -1179,20 +1199,20 @@ public class ZipFile implements ZipConstants, Closeable {
         private static final int[] EMPTY_META_VERSIONS = new int[0];
 
         private final Key key;               // the key in files
-        private final @Stable ZipCoder zc;   // zip coder used to decode/encode
+        private final @Stable ZipCoder zc;   // ZIP coder used to decode/encode
 
         private int refs = 1;
 
-        private RandomAccessFile zfile;      // zfile of the underlying zip file
+        private RandomAccessFile zfile;      // zfile of the underlying ZIP file
         private byte[] cen;                  // CEN & ENDHDR
         private long locpos;                 // position of first LOC header (usually 0)
-        private byte[] comment;              // zip file comment
+        private byte[] comment;              // ZIP file comment
                                              // list of meta entries in META-INF dir
         private int   manifestPos = -1;      // position of the META-INF/MANIFEST.MF, if exists
         private int   manifestNum = 0;       // number of META-INF/MANIFEST.MF, case insensitive
         private int[] signatureMetaNames;    // positions of signature related entries, if such exist
         private int[] metaVersions;          // list of unique versions found in META-INF/versions/
-        private final boolean startsWithLoc; // true, if zip file starts with LOCSIG (usually true)
+        private final boolean startsWithLoc; // true, if ZIP file starts with LOCSIG (usually true)
 
         // A Hashmap for all entries.
         //
@@ -1233,17 +1253,24 @@ public class ZipFile implements ZipConstants, Closeable {
             }
             int entryPos = pos + CENHDR;
             int nlen = CENNAM(cen, pos);
-            if (entryPos + nlen > cen.length - ENDHDR) {
+            int elen = CENEXT(cen, pos);
+            int clen = CENCOM(cen, pos);
+            long headerSize = (long)CENHDR + nlen + clen + elen;
+            // CEN header size + name length + comment length + extra length
+            // should not exceed 65,535 bytes per the PKWare APP.NOTE
+            // 4.4.10, 4.4.11, & 4.4.12.  Also check that current CEN header will
+            // not exceed the length of the CEN array
+            if (headerSize > 0xFFFF || pos + headerSize > cen.length - ENDHDR) {
                 zerror("invalid CEN header (bad header size)");
             }
 
-            int elen = CENEXT(cen, pos);
             if (elen > 0 && !DISABLE_ZIP64_EXTRA_VALIDATION) {
-                long extraStartingOffset = pos + CENHDR + nlen;
-                if ((int)extraStartingOffset != extraStartingOffset) {
-                    zerror("invalid CEN header (bad extra offset)");
-                }
-                checkExtraFields(pos, (int)extraStartingOffset, elen);
+                checkExtraFields(pos, entryPos + nlen, elen);
+            } else if (elen == 0 && (CENSIZ(cen, pos) == ZIP64_MAGICVAL
+                    || CENLEN(cen, pos) == ZIP64_MAGICVAL
+                    || CENOFF(cen, pos) == ZIP64_MAGICVAL
+                    || CENDSK(cen, pos) == ZIP64_MAGICCOUNT)) {
+                zerror("Invalid CEN header (invalid zip64 extra len size)");
             }
 
             try {
@@ -1256,10 +1283,9 @@ public class ZipFile implements ZipConstants, Closeable {
                 entries[index++] = hash;
                 entries[index++] = next;
                 entries[index  ] = pos;
-                // Validate comment if it exists
-                // if the bytes representing the comment cannot be converted to
+                // Validate comment if it exists.
+                // If the bytes representing the comment cannot be converted to
                 // a String via zcp.toString, an Exception will be thrown
-                int clen = CENCOM(cen, pos);
                 if (clen > 0) {
                     int start = entryPos + nlen + elen;
                     zcp.toString(cen, start, clen);
@@ -1272,6 +1298,7 @@ public class ZipFile implements ZipConstants, Closeable {
 
         /**
          * Validate the Zip64 Extra block fields
+         * @param cenPos The CEN offset for the current Entry
          * @param startingOffset Extra Field starting offset within the CEN
          * @param extraFieldLen Length of this Extra field
          * @throws ZipException  If an error occurs validating the Zip64 Extra
@@ -1286,7 +1313,7 @@ public class ZipFile implements ZipConstants, Closeable {
             }
             // CEN Offset where this Extra field ends
             int extraEndOffset = startingOffset + extraFieldLen;
-            if (extraEndOffset > cen.length) {
+            if (extraEndOffset > cen.length - ENDHDR) {
                 zerror("Invalid CEN header (extra data field size too long)");
             }
             int currentOffset = startingOffset;
@@ -1299,7 +1326,7 @@ public class ZipFile implements ZipConstants, Closeable {
 
                 int tagBlockSize = get16(cen, currentOffset);
                 currentOffset += Short.BYTES;
-                int tagBlockEndingOffset = currentOffset + tagBlockSize;
+                long tagBlockEndingOffset = (long)currentOffset + tagBlockSize;
 
                 //  The ending offset for this tag block should not go past the
                 //  offset for the end of the extra field
@@ -1315,9 +1342,13 @@ public class ZipFile implements ZipConstants, Closeable {
                     long csize = CENSIZ(cen, cenPos);
                     // Get the uncompressed size;
                     long size = CENLEN(cen, cenPos);
+                    // Get the LOC offset
+                    long locoff = CENOFF(cen, cenPos);
+                    // Get the Disk Number
+                    int diskNo = CENDSK(cen, cenPos);
 
                     checkZip64ExtraFieldValues(currentOffset, tagBlockSize,
-                            csize, size);
+                            csize, size, locoff, diskNo);
                 }
                 currentOffset += tagBlockSize;
             }
@@ -1325,25 +1356,29 @@ public class ZipFile implements ZipConstants, Closeable {
 
         /**
          * Validate the Zip64 Extended Information Extra Field (0x0001) block
-         * size and that the uncompressed size and compressed size field
-         * values are not negative.
-         * Note:  As we do not use the LOC offset or Starting disk number
-         * field value we will not validate them
+         * size; that the uncompressed size, compressed size field and LOC
+         * offset fields are not negative. Also make sure the field exists if
+         * the CEN header field is set to 0xFFFFFFFF.
+         * Note:  As we do not use the Starting disk number field,
+         * we will not validate its value
          * @param off the starting offset for the Zip64 field value
          * @param blockSize the size of the Zip64 Extended Extra Field
          * @param csize CEN header compressed size value
          * @param size CEN header uncompressed size value
+         * @param locoff CEN header LOC offset
+         * @param diskNo CEN header Disk number
          * @throws ZipException if an error occurs
          */
         private void checkZip64ExtraFieldValues(int off, int blockSize, long csize,
-                                                long size)
+                                                long size, long locoff, int diskNo)
                 throws ZipException {
             byte[] cen = this.cen;
             // if ZIP64_EXTID blocksize == 0, which may occur with some older
             // versions of Apache Ant and Commons Compress, validate csize and size
             // to make sure neither field == ZIP64_MAGICVAL
             if (blockSize == 0) {
-                if (csize == ZIP64_MAGICVAL || size == ZIP64_MAGICVAL) {
+                if (csize == ZIP64_MAGICVAL || size == ZIP64_MAGICVAL ||
+                        locoff == ZIP64_MAGICVAL || diskNo == ZIP64_MAGICCOUNT) {
                     zerror("Invalid CEN header (invalid zip64 extra data field size)");
                 }
                 // Only validate the ZIP64_EXTID data if the block size > 0
@@ -1351,22 +1386,45 @@ public class ZipFile implements ZipConstants, Closeable {
             }
             // Validate the Zip64 Extended Information Extra Field (0x0001)
             // length.
-            if (!isZip64ExtBlockSizeValid(blockSize)) {
+            if (!isZip64ExtBlockSizeValid(blockSize, csize, size, locoff, diskNo)) {
                 zerror("Invalid CEN header (invalid zip64 extra data field size)");
             }
             // Check the uncompressed size is not negative
-            // Note we do not need to check blockSize is >= 8 as
-            // we know its length is at least 8 from the call to
-            // isZip64ExtBlockSizeValid()
-            if ((size == ZIP64_MAGICVAL)) {
-                if(get64(cen, off) < 0) {
-                    zerror("Invalid zip64 extra block size value");
+            if (size == ZIP64_MAGICVAL) {
+                if ( blockSize >= Long.BYTES) {
+                    if (get64(cen, off) < 0) {
+                        zerror("Invalid zip64 extra block size value");
+                    }
+                    off += Long.BYTES;
+                    blockSize -= Long.BYTES;
+                } else {
+                    zerror("Invalid Zip64 extra block, missing size");
                 }
             }
             // Check the compressed size is not negative
-            if ((csize == ZIP64_MAGICVAL) && (blockSize >= 16)) {
-                if (get64(cen, off + 8) < 0) {
-                    zerror("Invalid zip64 extra block compressed size value");
+            if (csize == ZIP64_MAGICVAL) {
+                if (blockSize >= Long.BYTES) {
+                    if (get64(cen, off) < 0) {
+                        zerror("Invalid zip64 extra block compressed size value");
+                    }
+                    off += Long.BYTES;
+                    blockSize -= Long.BYTES;
+                } else {
+                    zerror("Invalid Zip64 extra block, missing compressed size");
+                }
+            }
+            // Check the LOC offset is not negative
+            if (locoff == ZIP64_MAGICVAL) {
+                if (blockSize >= Long.BYTES) {
+                    if (get64(cen, off) < 0) {
+                        zerror("Invalid zip64 extra block LOC OFFSET value");
+                    }
+                    // Note: We do not need to adjust the following fields as
+                    // this is the last field we are leveraging
+                    // off += Long.BYTES;
+                    // blockSize -= Long.BYTES;
+                } else {
+                    zerror("Invalid Zip64 extra block, missing LOC offset value");
                 }
             }
         }
@@ -1383,22 +1441,22 @@ public class ZipFile implements ZipConstants, Closeable {
          * See PKWare APP.Note Section 4.5.3 for more details
          *
          * @param blockSize the Zip64 Extended Information Extra Field size
+         * @param csize CEN header compressed size value
+         * @param size CEN header uncompressed size value
+         * @param locoff CEN header LOC offset
+         * @param diskNo CEN header Disk number
          * @return true if the extra block size is valid; false otherwise
          */
-        private static boolean isZip64ExtBlockSizeValid(int blockSize) {
-            /*
-             * As the fields must appear in order, the block size indicates which
-             * fields to expect:
-             *  8 - uncompressed size
-             * 16 - uncompressed size, compressed size
-             * 24 - uncompressed size, compressed sise, LOC Header offset
-             * 28 - uncompressed size, compressed sise, LOC Header offset,
-             * and Disk start number
-             */
-            return switch(blockSize) {
-                case 8, 16, 24, 28 -> true;
-                default -> false;
-            };
+        private static boolean isZip64ExtBlockSizeValid(int blockSize, long csize,
+                                                        long size, long locoff,
+                                                        int diskNo) {
+            int expectedBlockSize =
+                    (csize == ZIP64_MAGICVAL ? Long.BYTES : 0) +
+                    (size == ZIP64_MAGICVAL ? Long.BYTES : 0) +
+                    (locoff == ZIP64_MAGICVAL ? Long.BYTES : 0) +
+                    (diskNo == ZIP64_MAGICCOUNT ? Integer.BYTES : 0);
+            return expectedBlockSize == blockSize;
+
         }
         private int getEntryHash(int index) { return entries[index]; }
         private int getEntryNext(int index) { return entries[index + 1]; }
@@ -1408,6 +1466,13 @@ public class ZipFile implements ZipConstants, Closeable {
         private int[] table;                 // Hash chain heads: indexes into entries
         private int tablelen;                // number of hash heads
 
+        /**
+         * A class representing a key to a ZIP file. A key is based
+         * on the file key if available, or the path value if the
+         * file key is not available. The key is also based on the
+         * file's last modified time to allow for cases where a ZIP
+         * file is re-opened after it has been modified.
+         */
         private static class Key {
             final BasicFileAttributes attrs;
             File file;
@@ -1422,7 +1487,9 @@ public class ZipFile implements ZipConstants, Closeable {
             public int hashCode() {
                 long t = utf8 ? 0 : Long.MAX_VALUE;
                 t += attrs.lastModifiedTime().toMillis();
-                return ((int)(t ^ (t >>> 32))) + file.hashCode();
+                Object fk = attrs.fileKey();
+                return Long.hashCode(t) +
+                        (fk != null ? fk.hashCode() : file.hashCode());
             }
 
             public boolean equals(Object obj) {
@@ -1625,7 +1692,7 @@ public class ZipFile implements ZipConstants, Closeable {
                                 zerror("zip comment read failed");
                             }
                         }
-                        // must check for a zip64 end record; it is always permitted to be present
+                        // must check for a ZIP64 end record; it is always permitted to be present
                         try {
                             byte[] loc64 = new byte[ZIP64_LOCHDR];
                             if (end.endpos < ZIP64_LOCHDR ||
@@ -1654,7 +1721,7 @@ public class ZipFile implements ZipConstants, Closeable {
                             end.cenoff = cenoff64;
                             end.centot = (int)centot64; // assume total < 2g
                             end.endpos = end64pos;
-                        } catch (IOException x) {}    // no zip64 loc/end
+                        } catch (IOException x) {}    // no ZIP64 loc/end
                         return end;
                     }
                 }
@@ -1662,7 +1729,7 @@ public class ZipFile implements ZipConstants, Closeable {
             throw new ZipException("zip END header not found");
         }
 
-        // Reads zip file central directory.
+        // Reads ZIP file central directory.
         private void initCEN(int knownTotal) throws IOException {
             // Prefer locals for better performance during startup
             byte[] cen;
@@ -1679,7 +1746,7 @@ public class ZipFile implements ZipConstants, Closeable {
                     zerror("invalid END header (bad central directory size)");
                 long cenpos = end.endpos - end.cenlen;     // position of CEN table
                 // Get position of first local file (LOC) header, taking into
-                // account that there may be a stub prefixed to the zip file.
+                // account that there may be a stub prefixed to the ZIP file.
                 locpos = cenpos - end.cenoff;
                 if (locpos < 0) {
                     zerror("invalid END header (bad central directory offset)");
@@ -1722,7 +1789,7 @@ public class ZipFile implements ZipConstants, Closeable {
             manifestNum = 0;
             while (entryPos <= limit) {
                 if (idx >= entriesLength) {
-                    // This will only happen if the zip file has an incorrect
+                    // This will only happen if the ZIP file has an incorrect
                     // ENDTOT field, which usually means it contains more than
                     // 65535 entries.
                     initCEN(countCENHeaders(cen, limit));
@@ -1795,7 +1862,7 @@ public class ZipFile implements ZipConstants, Closeable {
         }
 
         /*
-         * Returns the {@code pos} of the zip cen entry corresponding to the
+         * Returns the {@code pos} of the ZIP cen entry corresponding to the
          * specified entry name, or -1 if not found.
          */
         private int getEntryPos(String name, boolean addSlash) {
@@ -1983,9 +2050,9 @@ public class ZipFile implements ZipConstants, Closeable {
 
         /**
          * Returns the number of CEN headers in a central directory.
-         * Will not throw, even if the zip file is corrupt.
+         * Will not throw, even if the ZIP file is corrupt.
          *
-         * @param cen copy of the bytes in a zip file's central directory
+         * @param cen copy of the bytes in a ZIP file's central directory
          * @param size number of bytes in central directory
          */
         private static int countCENHeaders(byte[] cen, int size) {
