@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -85,6 +85,11 @@ module java.base {
     exports java.io;
     exports java.lang;
     exports java.lang.annotation;
+    exports java.lang.classfile;
+    exports java.lang.classfile.attribute;
+    exports java.lang.classfile.components;
+    exports java.lang.classfile.constantpool;
+    exports java.lang.classfile.instruction;
     exports java.lang.constant;
     exports java.lang.foreign;
     exports java.lang.invoke;
@@ -156,13 +161,19 @@ module java.base {
     // module declaration be annotated with jdk.internal.javac.ParticipatesInPreview
     exports jdk.internal.javac to
         java.compiler,
+        java.desktop, // for ScopedValue
         jdk.compiler,
         jdk.incubator.vector, // participates in preview features
-        jdk.jshell;
+        jdk.jartool, // participates in preview features
+        jdk.jdeps, // participates in preview features
+        jdk.jfr, // participates in preview features
+        jdk.jlink,   // participates in preview features
+        jdk.jshell; // participates in preview features
     exports jdk.internal.access to
         java.desktop,
         java.logging,
         java.management,
+        java.management.rmi,
         java.naming,
         java.rmi,
         jdk.charsets,
@@ -195,19 +206,6 @@ module java.base {
         openjceplus,
 /*[ENDIF] OPENJCEPLUS_SUPPORT */
         java.logging;
-    exports jdk.internal.classfile to
-        jdk.jartool,
-        jdk.jlink,
-        jdk.jshell;
-    exports jdk.internal.classfile.attribute to
-        jdk.jartool,
-        jdk.jlink;
-    exports jdk.internal.classfile.constantpool to
-        jdk.jartool,
-        jdk.jlink;
-    exports jdk.internal.classfile.instruction to
-        jdk.jlink,
-        jdk.jshell;
     exports jdk.internal.org.objectweb.asm to
         jdk.jfr;
     exports jdk.internal.org.objectweb.asm.tree to
@@ -245,6 +243,7 @@ module java.base {
         java.instrument,
         java.management.rmi,
         jdk.jartool,
+        jdk.compiler,
         jdk.jfr,
         jdk.jlink,
         jdk.jpackage;
@@ -284,14 +283,13 @@ module java.base {
         jdk.jfr;
     exports jdk.internal.util.xml.impl to
         jdk.jfr;
-    exports jdk.internal.util.random to
-        jdk.random;
     exports jdk.internal.util to
         java.desktop,
         java.prefs,
         java.security.jgss,
         java.smartcardio,
         jdk.charsets,
+        jdk.internal.vm.ci,
         jdk.jlink,
         jdk.jpackage,
         jdk.net;
@@ -338,8 +336,7 @@ module java.base {
         java.sql.rowset;
     exports sun.security.action to
         java.desktop,
-        java.security.jgss,
-        jdk.crypto.ec;
+        java.security.jgss;
     exports sun.security.internal.interfaces to
 /*[IF OPENJCEPLUS_SUPPORT]*/
         openjceplus,
@@ -352,20 +349,17 @@ module java.base {
         jdk.crypto.cryptoki;
     exports sun.security.jca to
         java.smartcardio,
-        jdk.crypto.ec,
         jdk.crypto.cryptoki,
         jdk.naming.dns;
     exports sun.security.pkcs to
 /*[IF OPENJCEPLUS_SUPPORT]*/
         openjceplus,
 /*[ENDIF] OPENJCEPLUS_SUPPORT */
-        jdk.crypto.ec,
         jdk.jartool;
     exports sun.security.provider to
         java.rmi,
         java.security.jgss,
         jdk.crypto.cryptoki,
-        jdk.crypto.ec,
         jdk.security.auth;
     exports sun.security.provider.certpath to
         java.naming,
@@ -387,20 +381,14 @@ module java.base {
         java.security.sasl,
         java.smartcardio,
         java.xml.crypto,
-        jdk.crypto.ec,
         jdk.crypto.cryptoki,
         jdk.jartool,
         jdk.security.auth,
         jdk.security.jgss;
-    exports sun.security.util.math to
-        jdk.crypto.ec;
-    exports sun.security.util.math.intpoly to
-        jdk.crypto.ec;
     exports sun.security.x509 to
 /*[IF OPENJCEPLUS_SUPPORT]*/
         openjceplus,
 /*[ENDIF] OPENJCEPLUS_SUPPORT */
-        jdk.crypto.ec,
         jdk.crypto.cryptoki,
         jdk.jartool;
     exports sun.security.validator to
@@ -445,7 +433,6 @@ module java.base {
     uses java.time.chrono.AbstractChronology;
     uses java.time.chrono.Chronology;
     uses java.time.zone.ZoneRulesProvider;
-    uses java.util.random.RandomGenerator;
     uses java.util.spi.CalendarDataProvider;
     uses java.util.spi.CalendarNameProvider;
     uses java.util.spi.CurrencyNameProvider;
@@ -471,9 +458,4 @@ module java.base {
     provides java.nio.file.spi.FileSystemProvider with
         jdk.internal.jrtfs.JrtFileSystemProvider;
 
-    provides java.util.random.RandomGenerator with
-        java.security.SecureRandom,
-        java.util.Random,
-        java.util.SplittableRandom;
-
 }
