@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -140,6 +140,7 @@ final class ProviderConfig {
         return (provider != null);
     }
 
+    @Override
     public boolean equals(Object obj) {
         if (this == obj) {
             return true;
@@ -152,8 +153,9 @@ final class ProviderConfig {
 
     }
 
+    @Override
     public int hashCode() {
-        return provName.hashCode() + argument.hashCode();
+        return Objects.hash(provName, argument);
     }
 
     public String toString() {
@@ -192,57 +194,60 @@ final class ProviderConfig {
                 return null;
             }
 
-            // Create providers which are in java.base directly
-            if (provName.equals("SUN") || provName.equals("sun.security.provider.Sun")) {
-                p = new sun.security.provider.Sun();
-            } else if (provName.equals("SunRsaSign") || provName.equals("sun.security.rsa.SunRsaSign")) {
-                p = new sun.security.rsa.SunRsaSign();
-            } else if (provName.equals("SunJCE") || provName.equals("com.sun.crypto.provider.SunJCE")) {
-                p = new com.sun.crypto.provider.SunJCE();
-            } else if (provName.equals("SunJSSE")) {
-                p = new sun.security.ssl.SunJSSE();
-            } else if (provName.equals("Apple") || provName.equals("apple.security.AppleProvider")) {
-                // need to use reflection since this class only exists on MacOsx
-                @SuppressWarnings("removal")
-                var tmp = AccessController.doPrivileged(new PrivilegedAction<Provider>() {
-                    public Provider run() {
-                        try {
-                            Class<?> c = Class.forName("apple.security.AppleProvider");
-                            if (Provider.class.isAssignableFrom(c)) {
-                                @SuppressWarnings("deprecation")
-                                Object tmp = c.newInstance();
-                                return (Provider) tmp;
-                            } else {
+            p = switch (provName) {
+                case "SUN", "sun.security.provider.Sun" ->
+                    new sun.security.provider.Sun();
+                case "SunRsaSign", "sun.security.rsa.SunRsaSign" ->
+                    new sun.security.rsa.SunRsaSign();
+                case "SunJCE", "com.sun.crypto.provider.SunJCE" ->
+                    new com.sun.crypto.provider.SunJCE();
+                case "SunJSSE" -> new sun.security.ssl.SunJSSE();
+                case "SunEC" -> new sun.security.ec.SunEC();
+                case "Apple", "apple.security.AppleProvider" -> {
+                    // Reflection is needed for compile time as the class
+                    // is not available for non-macosx systems
+                    @SuppressWarnings("removal")
+                    var tmp = AccessController.doPrivileged(
+                        new PrivilegedAction<Provider>() {
+                            public Provider run() {
+                                try {
+                                    Class<?> c = Class.forName(
+                                        "apple.security.AppleProvider");
+                                    if (Provider.class.isAssignableFrom(c)) {
+                                        @SuppressWarnings("deprecation")
+                                        Object tmp = c.newInstance();
+                                        return (Provider) tmp;
+                                    }
+                                } catch (Exception ex) {
+                                    if (debug != null) {
+                                        debug.println("Error loading provider Apple");
+                                        ex.printStackTrace();
+                                    }
+                                }
                                 return null;
                             }
-                        } catch (Exception ex) {
-                            if (debug != null) {
-                                debug.println("Error loading provider Apple");
-                                ex.printStackTrace();
-                            }
-                            return null;
+                        });
+                    yield tmp;
+                }
+                default -> {
+                    if (isLoading) {
+                        // because this method is synchronized, this can only
+                        // happen if there is recursion.
+                        if (debug != null) {
+                            debug.println("Recursion loading provider: " + this);
+                            new Exception("Call trace").printStackTrace();
                         }
+                        yield null;
                     }
-                });
-                p = tmp;
-            } else {
-                if (isLoading) {
-                    // because this method is synchronized, this can only
-                    // happen if there is recursion.
-                    if (debug != null) {
-                        debug.println("Recursion loading provider: " + this);
-                        new Exception("Call trace").printStackTrace();
+                    try {
+                        isLoading = true;
+                        tries++;
+                        yield doLoadProvider();
+                    } finally {
+                        isLoading = false;
                     }
-                    return null;
                 }
-                try {
-                    isLoading = true;
-                    tries++;
-                    p = doLoadProvider();
-                } finally {
-                    isLoading = false;
-                }
-            }
+            };
             provider = p;
         }
         return p;
