@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,7 +25,7 @@
 
 /*
  * ===========================================================================
- * (c) Copyright IBM Corp. 2023, 2025 All Rights Reserved
+ * (c) Copyright IBM Corp. 2023, 2024 All Rights Reserved
  * ===========================================================================
  */
 
@@ -119,10 +119,6 @@ public final class ProviderList {
         if (!RestrictedSecurity.isProviderAllowed(p.getClass())) {
             // We're in restricted security mode which does not allow this provider,
             // return without adding.
-            if (debug != null) {
-                debug.println("In RestrictedSecurity mode. Provider " +
-                        p.getClass().getName() + " not allowed to be inserted.");
-            }
             return providerList;
         }
         if (providerList.getProvider(p.getName()) != null) {
@@ -408,7 +404,8 @@ public final class ProviderList {
                     continue;
                 }
                 Service s = p.getService(type, name);
-                if (s != null) {
+                if ((s != null) && RestrictedSecurity.isServiceAllowed(s)) {
+                    // We found a service that is allowed in restricted security mode.
                     return s;
                 }
             }
@@ -416,7 +413,8 @@ public final class ProviderList {
         for (i = 0; i < configs.length; i++) {
             Provider p = getProvider(i);
             Service s = p.getService(type, name);
-            if (s != null) {
+            if ((s != null) && RestrictedSecurity.isServiceAllowed(s)) {
+                // We found a service that is allowed in restricted security mode.
                 return s;
             }
         }
@@ -424,42 +422,28 @@ public final class ProviderList {
     }
 
     /**
-     * Return a List containing all the Services describing implementations
+     * Return an iterator over all the Services describing implementations
      * of the specified algorithms in precedence order. If no implementation
-     * exists, this method returns an empty List.
+     * exists, this method returns an empty iterator.
      *
-     * The elements of this list are determined lazily on demand.
+     * The elements of this iterator are determined lazily on demand.
      *
-     * The List returned is NOT thread safe.
+     * The iterator returned is NOT thread safe.
      */
-    public List<Service> getServices(String type, String algorithm) {
-        return new ServiceList(type, algorithm);
+    public Iterator<Service> getServices(String type, String algorithm) {
+        return new ServiceIterator(type, algorithm);
+    }
+
+    public Iterator<Service> getServices(List<ServiceId> ids) {
+        return new ServiceIterator(ids);
     }
 
     /**
-     * This method exists for compatibility with JCE only. It will be removed
-     * once JCE has been changed to use the replacement method.
-     * @deprecated use {@code getServices(List<ServiceId>)} instead
-     */
-    @Deprecated
-    public List<Service> getServices(String type, List<String> algorithms) {
-        List<ServiceId> ids = new ArrayList<>();
-        for (String alg : algorithms) {
-            ids.add(new ServiceId(type, alg));
-        }
-        return getServices(ids);
-    }
-
-    public List<Service> getServices(List<ServiceId> ids) {
-        return new ServiceList(ids);
-    }
-
-    /**
-     * Inner class for a List of Services. Custom List implementation in
+     * Inner class for an iterator over Services. Customized implementation in
      * order to delay Provider initialization and lookup.
      * Not thread safe.
      */
-    private final class ServiceList extends AbstractList<Service> {
+    private final class ServiceIterator implements Iterator<Service> {
 
         // type and algorithm for simple lookup
         // avoid allocating/traversing the ServiceId list for these lookups
@@ -482,17 +466,17 @@ public final class ProviderList {
         // index into config[] of the next provider we need to query
         private int providerIndex = 0;
 
-        // Matching preferred provider list for this ServiceList
+        // Matching preferred provider list for this ServiceIterator
         ArrayList<PreferredEntry> preferredList = null;
         private int preferredIndex = 0;
 
-        ServiceList(String type, String algorithm) {
+        ServiceIterator(String type, String algorithm) {
             this.type = type;
             this.algorithm = algorithm;
             this.ids = null;
         }
 
-        ServiceList(List<ServiceId> ids) {
+        ServiceIterator(List<ServiceId> ids) {
             this.type = null;
             this.algorithm = null;
             this.ids = ids;
@@ -552,14 +536,14 @@ public final class ProviderList {
                 if (type != null) {
                     // simple lookup
                     Service s = p.getService(type, algorithm);
-                    if (s != null) {
+                    if ((s != null) && RestrictedSecurity.isServiceAllowed(s)) {
                         addService(s);
                     }
                 } else {
                     // parallel lookup
                     for (ServiceId id : ids) {
                         Service s = p.getService(id.type, id.algorithm);
-                        if (s != null) {
+                        if ((s != null) && RestrictedSecurity.isServiceAllowed(s)) {
                             addService(s);
                         }
                     }
@@ -567,55 +551,26 @@ public final class ProviderList {
             }
         }
 
-        public Service get(int index) {
+        int index;
+
+        @Override
+        public boolean hasNext() {
+            return tryGet(index) != null;
+        }
+
+        @Override
+        public Service next() {
             Service s = tryGet(index);
             if (s == null) {
-                throw new IndexOutOfBoundsException();
+                throw new NoSuchElementException();
             }
+            index++;
             return s;
         }
 
-        public int size() {
-            int n;
-            if (services != null) {
-                n = services.size();
-            } else {
-                n = (firstService != null) ? 1 : 0;
-            }
-            while (tryGet(n) != null) {
-                n++;
-            }
-            return n;
-        }
-
-        // override isEmpty() and iterator() to not call size()
-        // this avoids loading + checking all Providers
-
-        public boolean isEmpty() {
-            return (tryGet(0) == null);
-        }
-
-        public Iterator<Service> iterator() {
-            return new Iterator<>() {
-                int index;
-
-                public boolean hasNext() {
-                    return tryGet(index) != null;
-                }
-
-                public Service next() {
-                    Service s = tryGet(index);
-                    if (s == null) {
-                        throw new NoSuchElementException();
-                    }
-                    index++;
-                    return s;
-                }
-
-                public void remove() {
-                    throw new UnsupportedOperationException();
-                }
-            };
+        @Override
+        public void remove() {
+            throw new UnsupportedOperationException();
         }
     }
 
@@ -625,9 +580,9 @@ public final class ProviderList {
 
         /*
          * Return a list of all preferred entries that match the passed
-         * ServiceList.
+         * ServiceIterator.
          */
-        ArrayList<PreferredEntry> getAll(ServiceList s) {
+        ArrayList<PreferredEntry> getAll(ServiceIterator s) {
             if (s.ids == null) {
                 return getAll(s.type, s.algorithm);
 
