@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -41,6 +41,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.EnumSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
@@ -229,17 +230,10 @@ enum SignatureScheme {
         }
     }
 
-    // Handshake signature scope.
-    static final Set<SSLScope> HANDSHAKE_SCOPE =
-            Set.of(SSLScope.HANDSHAKE_SIGNATURE);
-
-    // Certificate signature scope.
-    static final Set<SSLScope> CERTIFICATE_SCOPE =
-            Set.of(SSLScope.CERTIFICATE_SIGNATURE);
-
-    // Non-TLS specific SIGNATURE CryptoPrimitive.
+    // performance optimization
     private static final Set<CryptoPrimitive> SIGNATURE_PRIMITIVE_SET =
-            Set.of(CryptoPrimitive.SIGNATURE);
+        Collections.unmodifiableSet(EnumSet.of(CryptoPrimitive.SIGNATURE));
+
 
     SignatureScheme(int id, String name,
             String algorithm, String keyAlgorithm,
@@ -373,56 +367,24 @@ enum SignatureScheme {
         return 2;
     }
 
-    private boolean isPermitted(
-            SSLAlgorithmConstraints constraints, Set<SSLScope> scopes) {
-        return constraints.permits(this.name, scopes)
-                && constraints.permits(this.keyAlgorithm, scopes)
-                && constraints.permits(this.algorithm, scopes)
-                && constraints.permits(SIGNATURE_PRIMITIVE_SET, this.name, null)
-                && constraints.permits(SIGNATURE_PRIMITIVE_SET, this.keyAlgorithm, null)
-                && constraints.permits(SIGNATURE_PRIMITIVE_SET, this.algorithm,
-                (signAlgParams != null ? signAlgParams.parameters : null))
-                && (namedGroup == null || namedGroup.isPermitted(constraints));
-    }
-
-    // Helper method to update all locally supported signature schemes for
-    // a given HandshakeContext.
-    static void updateHandshakeLocalSupportedAlgs(HandshakeContext hc) {
-        // To improve performance we only update when necessary.
-        // No need to do anything if we already computed the local supported
-        // algorithms and either there is no negotiated protocol yet or the
-        // only active protocol ends up to be the negotiated protocol.
-        if (hc.localSupportedSignAlgs != null
-                && hc.localSupportedCertSignAlgs != null
-                && (hc.negotiatedProtocol == null
-                || hc.activeProtocols.size() == 1)) {
-            return;
-        }
-
-        List<ProtocolVersion> protocols = hc.negotiatedProtocol != null ?
-                List.of(hc.negotiatedProtocol) :
-                hc.activeProtocols;
-
-        hc.localSupportedSignAlgs = getSupportedAlgorithms(
-                hc.sslConfig,
-                hc.algorithmConstraints,
-                protocols,
-                HANDSHAKE_SCOPE);
-
-        hc.localSupportedCertSignAlgs = getSupportedAlgorithms(
-                hc.sslConfig,
-                hc.algorithmConstraints,
-                protocols,
-                CERTIFICATE_SCOPE);
+    private boolean isPermitted(AlgorithmConstraints constraints) {
+        return constraints.permits(SIGNATURE_PRIMITIVE_SET,
+                        this.name, null) &&
+               constraints.permits(SIGNATURE_PRIMITIVE_SET,
+                        this.keyAlgorithm, null) &&
+               constraints.permits(SIGNATURE_PRIMITIVE_SET,
+                        this.algorithm, (signAlgParams != null ?
+                                signAlgParams.parameters : null)) &&
+                        (namedGroup == null ||
+                            namedGroup.isPermitted(constraints));
     }
 
     // Get local supported algorithm collection complying to algorithm
-    // constraints and SSL scopes.
-    private static List<SignatureScheme> getSupportedAlgorithms(
+    // constraints.
+    static List<SignatureScheme> getSupportedAlgorithms(
             SSLConfiguration config,
-            SSLAlgorithmConstraints constraints,
-            List<ProtocolVersion> activeProtocols,
-            Set<SSLScope> scopes) {
+            AlgorithmConstraints constraints,
+            List<ProtocolVersion> activeProtocols) {
         List<SignatureScheme> supported = new LinkedList<>();
 
         // If config.signatureSchemes is non-null then it means that
@@ -466,14 +428,14 @@ enum SignatureScheme {
 
             boolean isMatch = false;
             for (ProtocolVersion pv : activeProtocols) {
-                if (ss.isSupportedProtocol(pv, scopes)) {
+                if (ss.supportedProtocols.contains(pv)) {
                     isMatch = true;
                     break;
                 }
             }
 
             if (isMatch) {
-                if (ss.isPermitted(constraints, scopes)) {
+                if (ss.isPermitted(constraints)) {
                     supported.add(ss);
                 } else if (SSLLogger.isOn &&
                         SSLLogger.isOn("ssl,handshake,verbose")) {
@@ -492,10 +454,8 @@ enum SignatureScheme {
 
     static List<SignatureScheme> getSupportedAlgorithms(
             SSLConfiguration config,
-            SSLAlgorithmConstraints constraints,
-            ProtocolVersion protocolVersion,
-            int[] algorithmIds,
-            Set<SSLScope> scopes) {
+            AlgorithmConstraints constraints,
+            ProtocolVersion protocolVersion, int[] algorithmIds) {
         List<SignatureScheme> supported = new LinkedList<>();
         for (int ssid : algorithmIds) {
             SignatureScheme ss = SignatureScheme.valueOf(ssid);
@@ -505,9 +465,11 @@ enum SignatureScheme {
                             "Unsupported signature scheme: " +
                             SignatureScheme.nameOf(ssid));
                 }
-            } else if ((config.signatureSchemes == null
-                        || Utilities.contains(config.signatureSchemes, ss.name))
-                    && ss.isAllowed(constraints, protocolVersion, scopes)) {
+            } else if (ss.isAvailable &&
+                    ss.supportedProtocols.contains(protocolVersion) &&
+                    (config.signatureSchemes == null ||
+                        Utilities.contains(config.signatureSchemes, ss.name)) &&
+                    ss.isPermitted(constraints)) {
                 supported.add(ss);
             } else {
                 if (SSLLogger.isOn && SSLLogger.isOn("ssl,handshake")) {
@@ -521,14 +483,16 @@ enum SignatureScheme {
     }
 
     static SignatureScheme getPreferableAlgorithm(
-            SSLAlgorithmConstraints constraints,
+            AlgorithmConstraints constraints,
             List<SignatureScheme> schemes,
             String keyAlgorithm,
             ProtocolVersion version) {
 
         for (SignatureScheme ss : schemes) {
-            if (keyAlgorithm.equalsIgnoreCase(ss.keyAlgorithm)
-                    && ss.isAllowed(constraints, version, HANDSHAKE_SCOPE)) {
+            if (ss.isAvailable &&
+                    ss.handshakeSupportedProtocols.contains(version) &&
+                    keyAlgorithm.equalsIgnoreCase(ss.keyAlgorithm) &&
+                    ss.isPermitted(constraints)) {
                 return ss;
             }
         }
@@ -538,7 +502,7 @@ enum SignatureScheme {
 
     static Map.Entry<SignatureScheme, Signature> getSignerOfPreferableAlgorithm(
             SSLConfiguration sslConfig,
-            SSLAlgorithmConstraints constraints,
+            AlgorithmConstraints constraints,
             List<SignatureScheme> schemes,
             X509Possession x509Possession,
             ProtocolVersion version) {
@@ -554,9 +518,10 @@ enum SignatureScheme {
             keySize = Integer.MAX_VALUE;
         }
         for (SignatureScheme ss : schemes) {
-            if (keySize >= ss.minimalKeySize &&
+            if (ss.isAvailable && (keySize >= ss.minimalKeySize) &&
+                    ss.handshakeSupportedProtocols.contains(version) &&
                     keyAlgorithm.equalsIgnoreCase(ss.keyAlgorithm) &&
-                    ss.isAllowed(constraints, version, HANDSHAKE_SCOPE)) {
+                    ss.isPermitted(constraints)) {
                 if ((ss.namedGroup != null) && (ss.namedGroup.spec ==
                         NamedGroupSpec.NAMED_GROUP_ECDHE)) {
                     ECParameterSpec params =
@@ -616,26 +581,6 @@ enum SignatureScheme {
         return null;
     }
 
-    // Returns true if this signature scheme is supported for the given
-    // protocol version and SSL scopes.
-    private boolean isSupportedProtocol(
-            ProtocolVersion version, Set<SSLScope> scopes) {
-        if (scopes != null && scopes.equals(HANDSHAKE_SCOPE)) {
-            return this.handshakeSupportedProtocols.contains(version);
-        } else {
-            return this.supportedProtocols.contains(version);
-        }
-    }
-
-    // Returns true if this signature scheme is available, supported and
-    // permitted for the given constraints, protocol version and SSL scopes.
-    private boolean isAllowed(SSLAlgorithmConstraints constraints,
-            ProtocolVersion version, Set<SSLScope> scopes) {
-        return isAvailable
-                && isSupportedProtocol(version, scopes)
-                && isPermitted(constraints, scopes);
-    }
-
     static String[] getAlgorithmNames(Collection<SignatureScheme> schemes) {
         if (schemes != null) {
             ArrayList<String> names = new ArrayList<>(schemes.size());
