@@ -154,6 +154,7 @@ Java_sun_security_pkcs11_wrapper_PKCS11_getNativeKeyInfo
     jbyte* nativeKeyInfoArrayRaw = NULL;
     jbyte* nativeKeyInfoWrappedKeyArrayRaw = NULL;
     unsigned int sensitiveAttributePosition = (unsigned int)-1;
+    unsigned int valueLenAttributePosition = (unsigned int)-1;
     unsigned int i = 0U;
     unsigned long totalDataSize = 0UL, attributesCount = 0UL;
     unsigned long totalCkAttributesSize = 0UL, totalNativeKeyInfoArraySize = 0UL;
@@ -223,6 +224,8 @@ Java_sun_security_pkcs11_wrapper_PKCS11_getNativeKeyInfo
             if ((ckpAttributes+i)->type == CKA_SENSITIVE) {
                  sensitiveAttributePosition = attributesCount;
                  TRACE0("DEBUG: GetNativeKeyInfo key is sensitive");
+            } else if ((ckpAttributes+i)->type == CKA_VALUE_LEN) {
+                valueLenAttributePosition = attributesCount;
             }
             attributesCount++;
         }
@@ -302,6 +305,14 @@ Java_sun_security_pkcs11_wrapper_PKCS11_getNativeKeyInfo
         goto cleanup;
     }
 
+    if (class == CKO_SECRET_KEY && (valueLenAttributePosition != (unsigned int)-1) &&
+            *(CK_ULONG*)(((CK_ATTRIBUTE_PTR)(((CK_ATTRIBUTE_PTR)nativeKeyInfoArrayRawCkAttributes)
+                    +valueLenAttributePosition))->pValue) > 256UL) {
+        // NSS' NSC_UnwrapKey does not accept CKO_SECRET_KEY keys with length greater
+        // than 256 (MAX_KEY_LEN - pkcs11i.h). Handle these keys as non-extractable.
+        goto cleanup;
+    }
+
     TRACE0("DEBUG: GetNativeKeyInfo 1st C_GetAttributeValue call passed\n");
 
     if (netscapeAttributeValueNeeded) {
